@@ -9,6 +9,7 @@ http://code.activestate.com/recipes/437932/.
 """
 
 import inspect
+import os
 import optparse
 import re
 import sys
@@ -17,20 +18,19 @@ import sys
 class Action(object):
     """Represents a single action to be applied to each line."""
 
-    def __init__(self, pattern=None, cmd='l', statement=False, negate=False):
+    def __init__(self, pattern=None, cmd='l', statement=False, negate=False, strict=False):
         self.delim = None
         self.odelim = ' '
         self.negate = negate
         self.pattern = None if pattern is None else re.compile(pattern)
         self.cmd = cmd
+        self.strict = strict
         self._compile(statement)
 
     @classmethod
     def from_options(cls, options, arg):
-        self = cls()
-        self.negate, self.pattern, self.cmd = self._parse_command(arg)
-        self._compile(options.statement)
-        return self
+        negate, pattern, cmd = Action._parse_command(arg)
+        return cls(pattern=pattern, cmd=cmd, statement=options.statement, negate=negate, strict=options.strict)
 
     def _compile(self, statement):
         if not self.cmd:
@@ -40,13 +40,30 @@ class Action(object):
                 self.cmd = 'l'
         self._codeobj = compile(self.cmd, 'EXPR', 'exec' if statement else 'eval')
 
-    def apply(self, context, numz, line):
-        """Apply action to line."""
+    def apply(self, context, line):
+        """Apply action to line.
+
+        :return: Line text or None.
+        """
         match = self._match(line)
         if match is None:
             return None
         context['m'] = match
-        return eval(self._codeobj, globals(), context)
+        try:
+            result = eval(self._codeobj, globals(), context)
+        except:
+            if self.strict:
+                raise
+            return None
+        if result is None or result is False:
+            return None
+        elif result is True:
+            result = line
+        elif isinstance(result, (list, tuple)):
+            result = context.odelim.join(map(str, result))
+        else:
+            result = str(result)
+        return result
 
     def _match(self, line):
         if self.pattern is None:
@@ -57,20 +74,19 @@ class Action(object):
         elif self.negate:
             return ()
 
-    def _parse_command(self, arg):
+    @staticmethod
+    def _parse_command(arg):
         match = re.match(r'(?:(!)?/((?:\\.|[^/])+)/)?(.*)', arg)
         negate, pattern, cmd = match.groups()
         cmd = cmd.strip()
         negate = bool(negate)
-        if pattern is not None:
-            pattern = re.compile(pattern)
         return negate, pattern, cmd
 
 
 class Context(dict):
     def apply(self, numz, line):
-        l = line.strip()
-        f = [w for w in l.split(self.delim) if w]
+        l = line.rstrip()
+        f = tuple([w for w in l.split(self.delim) if w])
         self.update(line=line, l=l, n=numz + 1, f=f, nf=len(f))
 
     @classmethod
@@ -111,23 +127,11 @@ def process(context, input, output, begin_statement, actions, end_statement, str
         for numz, line in enumerate(input):
             context.apply(numz, line)
             for action in actions:
-                try:
-                    result = action.apply(context, numz, line)
-                except:
-                    if strict:
-                        raise
-                    continue
-                if result is None or result is False:
-                    continue
-                elif result is True:
-                    result = line
-                elif isinstance(result, (list, tuple)):
-                    result = context.odelim.join(map(str, result))
-                else:
-                    result = str(result)
-                write(result)
-                if not result.endswith('\n'):
-                    write('\n')
+                result = action.apply(context, line)
+                if result is not None:
+                    write(result)
+                    if not result.endswith('\n'):
+                        write('\n')
 
         if end_statement:
             end = compile(end_statement, 'END', 'single')
@@ -136,10 +140,10 @@ def process(context, input, output, begin_statement, actions, end_statement, str
         sys.stdout = old_stdout
 
 
-# For integration tests.
-def run(argv, input, output):
+def parse_commandline(argv):
     parser = optparse.OptionParser()
     parser.set_usage(__doc__.strip())
+    parser.add_option('-I', '--in_place', dest='in_place', help='modify given input file in-place', metavar='<filename>')
     parser.add_option('-i', '--import', dest='imports', help='comma-separated list of modules to "from x import *" from', metavar='<modules>')
     parser.add_option('-F', dest='delim', help='input delimiter', metavar='<delim>', default=None)
     parser.add_option('-O', dest='delim_out', help='output delimiter', metavar='<delim>', default=' ')
@@ -147,8 +151,17 @@ def run(argv, input, output):
     parser.add_option('-E', '--end', help='end statement', metavar='<statement>')
     parser.add_option('-s', '--statement', action='store_true', help='execute <expr> as a statement instead of an expression')
     parser.add_option('--strict', action='store_true', help='abort on exceptions')
+    return parser.parse_args(argv[1:])
 
-    options, args = parser.parse_args(argv[1:])
+
+# For integration tests.
+def run(argv, input, output):
+    options, args = parse_commandline(argv)
+
+    if options.in_place:
+        os.rename(options.in_place, options.in_place + '~')
+        input = open(options.in_place + '~')
+        output = open(options.in_place, 'w')
 
     # Auto-import. This is not smart.
     all_text = ' '.join([(options.begin or ''), ' '.join(args), (options.end or '')])
@@ -159,7 +172,12 @@ def run(argv, input, output):
     if not actions:
         actions = [Action.from_options(options, '')]
 
-    process(context, input, output, options.begin, actions, options.end, options.strict)
+    try:
+        process(context, input, output, options.begin, actions, options.end, options.strict)
+    finally:
+        if options.in_place:
+            output.close()
+            input.close()
 
 
 def main():
